@@ -1,3 +1,4 @@
+import re
 from collections import OrderedDict
 from collections.abc import Iterable as IterableABC
 from datetime import datetime, time
@@ -7,26 +8,31 @@ from functools import partial as partial_type, reduce
 from operator import __or__
 from pathlib import Path
 from pprint import pformat
-from typing import (
-    Dict, Any, Optional, Sequence, Generator, TypeVar, List, Mapping, Pattern, Union,
-    Callable, Iterable
-)
 from types import GeneratorType
-import re
-
-from testfixtures import not_there, singleton
-from testfixtures.compat import PY_311_PLUS
-from testfixtures.resolve import resolve
-from testfixtures.utils import indent
-from testfixtures.mock import parent_name, mock_call
+from typing import (
+    Any,
+    Sequence,
+    TypeVar,
+    List,
+    Mapping,
+    Pattern,
+    Callable,
+    Iterable,
+    cast,
+    overload,
+)
 from unittest.mock import call as unittest_mock_call
 
+from testfixtures import not_there, singleton
+from testfixtures.mock import parent_name, mock_call, _Call
+from testfixtures.resolve import resolve
+from testfixtures.utils import indent
 
 # Some common types that are immutable, for optimisation purposes within CompareContext
 IMMUTABLE_TYPEs = str, bytes, int, float, tuple, type(None)
 
 
-def diff(x: str, y: str, x_label: str = '', y_label: str = ''):
+def diff(x: str, y: str, x_label: str | None = '', y_label: str | None = '') -> str:
     """
     A shorthand function that uses :mod:`difflib` to return a
     string representing the differences between the two string
@@ -44,7 +50,7 @@ def diff(x: str, y: str, x_label: str = '', y_label: str = ''):
     )
 
 
-def compare_simple(x, y, context: 'CompareContext'):
+def compare_simple(x: Any, y: Any, context: 'CompareContext') -> str | None:
     """
     Returns a very simple textual difference between the two supplied objects.
     """
@@ -66,9 +72,10 @@ def compare_simple(x, y, context: 'CompareContext'):
                 context.x_label or 'x', context.y_label or 'y', repr_x
             )
         return context.label('x', repr_x) + ' != ' + context.label('y', repr_y)
+    return None
 
 
-def _extract_attrs(obj, ignore: Iterable[str] = None) -> Optional[Dict[str, Any]]:
+def _extract_attrs(obj: Any, ignore: Iterable[str] | None = None) -> dict[str, Any] | None:
     try:
         attrs = vars(obj).copy()
     except TypeError:
@@ -79,7 +86,7 @@ def _extract_attrs(obj, ignore: Iterable[str] = None) -> Optional[Dict[str, Any]
 
     has_slots = getattr(obj, '__slots__', not_there) is not not_there
     if has_slots:
-        slots = set()
+        slots = set[str]()
         for cls in type(obj).__mro__:
             slots.update(getattr(cls, '__slots__', ()))
         if slots and attrs is None:
@@ -99,7 +106,7 @@ def _extract_attrs(obj, ignore: Iterable[str] = None) -> Optional[Dict[str, Any]
 
 
 def _attrs_to_ignore(
-        context: 'CompareContext', ignore_attributes: Iterable[str], obj
+        context: 'CompareContext', ignore_attributes: Iterable[str], obj: Any
 ) -> Iterable[str]:
     ignore = context.get_option('ignore_attributes', ())
     if isinstance(ignore, dict):
@@ -110,8 +117,8 @@ def _attrs_to_ignore(
 
 
 def compare_object(
-        x, y, context: 'CompareContext', ignore_attributes: Iterable[str] = ()
-) -> Optional[str]:
+        x: object, y: object, context: 'CompareContext', ignore_attributes: Iterable[str] = ()
+) -> str | None:
     """
     Compare the two supplied objects based on their type and attributes.
 
@@ -136,11 +143,12 @@ def compare_object(
     if not context.simple_equals(x_attrs, y_attrs):
         return _compare_mapping(x_attrs, y_attrs, context, x,
                                 'attributes ', '.%s')
+    return None
 
 
 def compare_exception(
         x: BaseException, y: BaseException, context: 'CompareContext'
-) -> Optional[str]:
+) -> str | None:
     """
     Compare the two supplied exceptions based on their message, type and
     attributes.
@@ -150,7 +158,7 @@ def compare_exception(
     return compare_object(x, y, context)
 
 
-def compare_with_type(x, y, context: 'CompareContext') -> str:
+def compare_with_type(x: Any, y: Any, context: 'CompareContext') -> str:
     """
     Return a textual description of the difference between two objects
     including information about their types.
@@ -170,7 +178,7 @@ def compare_with_type(x, y, context: 'CompareContext') -> str:
 
 def compare_sequence(
         x: Sequence, y: Sequence, context: 'CompareContext', prefix: bool = True
-) -> Optional[str]:
+) -> str | None:
     """
     Returns a textual description of the differences between the two
     supplied sequences.
@@ -184,7 +192,7 @@ def compare_sequence(
         i += 1
 
     if l_x == l_y and i == l_x:
-        return
+        return None
 
     return (('sequence not as expected:\n\n' if prefix else '')+
             'same:\n%s\n\n'
@@ -195,7 +203,7 @@ def compare_sequence(
                           )
 
 
-def compare_generator(x: Generator, y: Generator, context: 'CompareContext') -> Optional[str]:
+def compare_generator(x: Iterable, y: Iterable, context: 'CompareContext') -> str | None:
     """
     Returns a textual description of the differences between the two
     supplied generators.
@@ -208,12 +216,12 @@ def compare_generator(x: Generator, y: Generator, context: 'CompareContext') ->
     y = tuple(y)
 
     if context.simple_equals(x, y):
-        return
+        return None
 
     return compare_sequence(x, y, context)
 
 
-def compare_tuple(x: tuple, y: tuple, context: 'CompareContext') -> Optional[str]:
+def compare_tuple(x: tuple, y: tuple, context: 'CompareContext') -> str | None:
     """
     Returns a textual difference between two tuples or
     :func:`collections.namedtuple` instances.
@@ -234,7 +242,7 @@ def compare_tuple(x: tuple, y: tuple, context: 'CompareContext') -> Optional[str
     return compare_sequence(x, y, context)
 
 
-def compare_dict(x: dict, y: dict, context: 'CompareContext') -> Optional[str]:
+def compare_dict(x: dict, y: dict, context: 'CompareContext') -> str | None:
     """
     Returns a textual description of the differences between the two
     supplied dictionaries.
@@ -253,7 +261,7 @@ def _compare_mapping(
         x: Mapping, y: Mapping, context: 'CompareContext', obj_for_class: Any,
         prefix: str = '', breadcrumb: str = '[%r]',
         check_y_not_x: bool = True
-) -> Optional[str]:
+) -> str | None:
 
     x_keys = set(x.keys())
     y_keys = set(y.keys())
@@ -272,7 +280,7 @@ def _compare_mapping(
             same.append(key)
 
     if not (x_not_y or (check_y_not_x and y_not_x) or diffs):
-        return
+        return None
 
     if obj_for_class is not_there:
         lines = []
@@ -309,7 +317,7 @@ def _compare_mapping(
     return '\n'.join(lines)
 
 
-def compare_set(x: set, y: set, context: 'CompareContext') -> Optional[str]:
+def compare_set(x: set, y: set, context: 'CompareContext') -> str | None:
     """
     Returns a textual description of the differences between the two
     supplied sets.
@@ -317,7 +325,7 @@ def compare_set(x: set, y: set, context: 'CompareContext') -> Optional[str]:
     x_not_y = x - y
     y_not_x = y - x
     if not (y_not_x or x_not_y):
-        return
+        return None
     lines = ['%s not as expected:' % x.__class__.__name__, '']
     x_label = context.x_label or 'first'
     y_label = context.y_label or 'second'
@@ -355,7 +363,7 @@ def split_repr(text: str) -> str:
     return '\n'.join(parts)
 
 
-def compare_text(x: str, y: str, context: 'CompareContext'):
+def compare_text(x: str, y: str, context: 'CompareContext') -> str | None:
     """
     Returns an informative string describing the differences between the two
     supplied strings. The way in which this comparison is performed
@@ -384,7 +392,7 @@ def compare_text(x: str, y: str, context: 'CompareContext'):
         x = strip_blank_lines(x)
         y = strip_blank_lines(y)
     if x == y:
-        return
+        return None
     labelled_x = context.label('x', repr(x))
     labelled_y = context.label('y', repr(y))
     if len(x) > 10 or len(y) > 10:
@@ -400,19 +408,19 @@ def compare_text(x: str, y: str, context: 'CompareContext'):
     return message
 
 
-def compare_bytes(x: bytes, y: bytes, context: 'CompareContext') -> Optional[str]:
+def compare_bytes(x: bytes, y: bytes, context: 'CompareContext') -> str | None:
     if x == y:
-        return
+        return None
     labelled_x = context.label('x', repr(x))
     labelled_y = context.label('y', repr(y))
     return '\n%s\n!=\n%s' % (labelled_x, labelled_y)
 
 
-def compare_call(x, y, context: 'CompareContext') -> Optional[str]:
+def compare_call(x: _Call, y: _Call, context: 'CompareContext') -> str | None:
     if x == y:
-        return
+        return None
 
-    def extract(call):
+    def extract(call: _Call) -> tuple[str, tuple[Any], dict[str, Any]]:
         try:
             name, args, kwargs = call
         except ValueError:
@@ -435,24 +443,25 @@ def compare_call(x, y, context: 'CompareContext') -> Optional[str]:
         context.different(x_kw, y_kw, ' kw')
     )
     if not different:
-        return
+        return None
 
     return 'mock.call not as expected:'
 
 
-def compare_partial(x: partial_type, y: partial_type, context: 'CompareContext') -> Optional[str]:
+def compare_partial(x: partial_type, y: partial_type, context: 'CompareContext') -> str | None:
     x_attrs = dict(func=x.func, args=x.args, keywords=x.keywords)
     y_attrs = dict(func=y.func, args=y.args, keywords=y.keywords)
     if x_attrs != y_attrs:
         return _compare_mapping(x_attrs, y_attrs, context, x,
                                 'attributes ', '.%s')
+    return None
 
 
-def compare_path(x: Path, y: Path, context: 'CompareContext') -> Optional[str]:
+def compare_path(x: Path, y: Path, context: 'CompareContext') -> str | None:
     return compare_text(str(x), str(y), context)
 
 
-def compare_with_fold(x: datetime, y: datetime, context: 'CompareContext') -> Optional[str]:
+def compare_with_fold(x: datetime, y: datetime, context: 'CompareContext') -> str | None:
     if not (x == y and x.fold == y.fold):
         repr_x = repr(x)
         repr_y = repr(y)
@@ -460,17 +469,17 @@ def compare_with_fold(x: datetime, y: datetime, context: 'CompareContext') -> Op
             repr_x += f' (fold={x.fold})'
             repr_y += f' (fold={y.fold})'
         return context.label('x', repr_x)+' != '+context.label('y', repr_y)
+    return None
 
-
-def _short_repr(obj) -> str:
+def _short_repr(obj: Any) -> str:
     repr_ = repr(obj)
     if len(repr_) > 30:
         repr_ = repr_[:30] + '...'
     return repr_
 
 
-Comparer = Callable[[Any, Any, 'CompareContext'], Optional[str]]
-Registry = Dict[type, Comparer]
+Comparer = Callable[[Any, Any, 'CompareContext'], str | None]
+Registry = dict[type, Comparer]
 
 _registry: Registry = {
     dict: compare_dict,
@@ -492,26 +501,27 @@ _registry: Registry = {
     time: compare_with_fold,
 }
 
-if PY_311_PLUS:
 
-    def compare_exception_group(
-        x: BaseExceptionGroup, y: BaseExceptionGroup, context: 'CompareContext'
-    ) -> Optional[str]:
-        """
-        Compare the two supplied exception groups
-        """
+def compare_exception_group(
+    x: BaseExceptionGroup, y: BaseExceptionGroup, context: 'CompareContext'
+) -> str | None:
+    """
+    Compare the two supplied exception groups
+    """
 
-        x_msg, x_excs = x.args
-        y_msg, y_excs = y.args
-        msg_different = context.different(x_msg, y_msg, 'msg')
-        excs_different = context.different(x_excs, y_excs, 'excs')
-        if msg_different or excs_different:
-            return 'exception group not as expected:'
-
-    _registry[BaseExceptionGroup] = compare_exception_group
+    x_msg, x_excs = x.args
+    y_msg, y_excs = y.args
+    msg_different = context.different(x_msg, y_msg, 'msg')
+    excs_different = context.different(x_excs, y_excs, 'excs')
+    if msg_different or excs_different:
+        return 'exception group not as expected:'
+    return None
 
 
-def register(type_: type, comparer: Comparer):
+_registry[BaseExceptionGroup] = compare_exception_group
+
+
+def register(type_: type, comparer: Comparer) -> None:
     """
     Register the supplied comparer for the specified type.
     This registration is global and will be in effect from the point
@@ -520,7 +530,7 @@ def register(type_: type, comparer: Comparer):
     _registry[type_] = comparer
 
 
-def _shared_mro(x, y):
+def _shared_mro(x: Any, y: Any) -> Iterable[type]:
     y_mro = set(type(y).__mro__)
     for class_ in type(x).__mro__:
         if class_ in y_mro:
@@ -532,15 +542,15 @@ _unsafe_iterables = str, bytes, dict
 
 class AlreadySeen:
 
-    def __init__(self, id_, obj, breadcrumb):
+    def __init__(self, id_: int, obj: Any, breadcrumb: str) -> None:
         self.id = id_
         self.obj = obj
         self.breadcrumb = breadcrumb
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f'<AlreadySeen for {self.obj!r} at {self.breadcrumb} with id {self.id}>'
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any)-> bool:
         if isinstance(other, AlreadySeen):
             return self.breadcrumb == other.breadcrumb
         else:
@@ -555,13 +565,13 @@ class CompareContext:
 
     def __init__(
             self,
-            x_label: Optional[str],
-            y_label: Optional[str],
+            x_label: str | None,
+            y_label: str | None,
             recursive: bool = True,
             strict: bool = False,
             ignore_eq: bool = False,
-            comparers: Registry = None,
-            options: Dict[str, Any] = None,
+            comparers: Registry | None = None,
+            options: dict[str, Any] | None = None,
     ):
         self.registries = []
         if comparers:
@@ -573,16 +583,16 @@ class CompareContext:
         self.recursive: bool = recursive
         self.strict: bool = strict
         self.ignore_eq: bool = ignore_eq
-        self.options: Dict[str, Any] = options or {}
+        self.options: dict[str, Any] = options or {}
         self.message: str = ''
         self.breadcrumbs: List[str] = []
-        self._seen = {}
+        self._seen: dict[int, str] = {}
 
     def extract_args(self, args: tuple, x: Any, y: Any, expected: Any, actual: Any) -> List:
 
-        possible = []
+        possible = list[Any]()
 
-        def append_if_specified(source):
+        def append_if_specified(source: Any) -> None:
             if source is not unspecified:
                 possible.append(source)
 
@@ -602,7 +612,7 @@ class CompareContext:
 
         return possible
 
-    def get_option(self, name: str, default=None):
+    def get_option(self, name: str, default: Any = None) -> Any:
         return self.options.get(name, default)
 
     def label(self, side: str, value: Any) -> str:
@@ -637,7 +647,7 @@ class CompareContext:
     def _separator(self) -> str:
         return '\n\nWhile comparing %s: ' % ''.join(self.breadcrumbs[1:])
 
-    def _break_loops(self, obj, breadcrumb):
+    def _break_loops(self, obj: Any, breadcrumb: str) -> Any:
         # Don't bother with this process for simple, immutable types:
         if isinstance(obj, IMMUTABLE_TYPEs):
             return obj
@@ -653,7 +663,7 @@ class CompareContext:
     def simple_equals(self, x: Any, y: Any) -> bool:
         return not (self.strict or self.ignore_eq) and x == y
 
-    def different(self, x: Any, y: Any, breadcrumb: str) -> Union[bool, Optional[str]]:
+    def different(self, x: Any, y: Any, breadcrumb: str) -> bool | str | None:
 
         x = self._break_loops(x, breadcrumb)
         y = self._break_loops(y, breadcrumb)
@@ -695,7 +705,7 @@ class CompareContext:
             self.breadcrumbs.pop()
 
 
-def _resolve_lazy(source):
+def _resolve_lazy(source: Any) -> str:
     return str(source() if callable(source) else source)
 
 
@@ -703,22 +713,22 @@ unspecified = singleton('unspecified')
 
 
 def compare(
-        *args,
+        *args: Any,
         x: Any = unspecified,
         y: Any = unspecified,
         expected: Any = unspecified,
         actual: Any = unspecified,
-        prefix: str = None,
-        suffix: str = None,
-        x_label: str = None,
-        y_label: str = None,
+        prefix: str | None = None,
+        suffix: str | None = None,
+        x_label: str | None = None,
+        y_label: str | None = None,
         raises: bool = True,
         recursive: bool = True,
         strict: bool = False,
         ignore_eq: bool = False,
-        comparers: Registry = None,
+        comparers: Registry | None = None,
         **options: Any
-) -> Optional[str]:
+) -> str | None:
     """
     Compare two objects, raising an :class:`AssertionError` if they are not
     the same. The :class:`AssertionError` raised will attempt to provide
@@ -787,7 +797,7 @@ def compare(
     context = CompareContext(x_label, y_label, recursive, strict, ignore_eq, comparers, options)
     x, y = context.extract_args(args, x, y, expected, actual)
     if not context.different(x, y, ''):
-        return
+        return None
 
     message = context.message
     if prefix:
@@ -805,11 +815,11 @@ class StatefulComparison:
     A base class for stateful comparison objects.
     """
 
-    failed: str = ''
+    failed: str | None = ''
     expected: Any = None
     name_attrs: Sequence[str] = ()
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         return not(self != other)
 
     def name(self) -> str:
@@ -853,8 +863,8 @@ class Comparison(StatefulComparison):
     """
 
     def __init__(self,
-                 object_or_type,
-                 attribute_dict: Dict[str, Any] = None,
+                 object_or_type: Any,
+                 attribute_dict: dict[str, Any] | None = None,
                  partial: bool = False,
                  **attributes: Any):
         self.partial = partial
@@ -887,10 +897,11 @@ class Comparison(StatefulComparison):
             return False
 
         attribute_names = set(self.expected_attributes.keys())
+        actual_attributes: dict[str, Any]
         if self.partial:
             actual_attributes = {}
         else:
-            actual_attributes = _extract_attrs(other)
+            actual_attributes = cast(dict[str, Any], _extract_attrs(other))
             attribute_names -= set(actual_attributes)
 
         for name in attribute_names:
@@ -949,18 +960,23 @@ class SequenceComparison(StatefulComparison):
       Defaults to ``False``.
     """
 
-    name_attrs = ('ordered', 'partial')
+    name_attrs: Sequence[str] = ('ordered', 'partial')
 
     def __init__(
-            self, *expected, ordered: bool = True, partial: bool = False, recursive: bool = False
+            self,
+            *expected: Any,
+            ordered: bool = True,
+            partial: bool = False,
+            recursive: bool = False,
     ):
         self.expected = expected
         self.ordered = ordered
         self.partial = partial
         self.recursive = recursive
-        self.checked_indices = set()
+        self.checked_indices = set[int]()
 
-    def __ne__(self, other) -> bool:
+    def __ne__(self, other: Any) -> bool:
+        actual: list[Any]
         try:
             actual = original_actual = list(other)
         except TypeError:
@@ -1012,26 +1028,26 @@ class SequenceComparison(StatefulComparison):
                 missing_from_expected.pop(0)
 
             ignored = missing_from_expected
-            missing_from_expected = None
+            missing_from_expected = []
         else:
             actual_indices += missing_from_expected_indices
             ignored = None
 
         message = []
 
-        def add_section(name, content):
+        def add_section(name: str, content: Any) -> None:
             if content:
                 message.append(name+':\n'+pformat(content))
 
         add_section('ignored', ignored)
 
         if self.ordered:
-            message.append(compare(
+            message.append(cast(str, compare(
                 expected=[self.expected[i] for i in sorted(expected_indices)],
                 actual=[original_actual[i] for i in sorted(actual_indices)],
                 recursive=self.recursive,
                 raises=False
-            ).split('\n\n', 1)[1])
+            )).split('\n\n', 1)[1])
         else:
             add_section('same', matched)
             add_section('in expected but not actual', missing_from_actual)
@@ -1049,7 +1065,7 @@ class Subset(SequenceComparison):
 
     name_attrs = ()
 
-    def __init__(self, *expected):
+    def __init__(self, *expected: Any) -> None:
         super(Subset, self).__init__(*expected, ordered=False, partial=True)
 
 
@@ -1059,7 +1075,7 @@ class Permutation(SequenceComparison):
     in the sequence is as expected, but without checking ordering.
     """
 
-    def __init__(self, *expected):
+    def __init__(self, *expected: Any) -> None:
         super(Permutation, self).__init__(*expected, ordered=False, partial=False)
 
 
@@ -1086,22 +1102,27 @@ class MappingComparison(StatefulComparison):
 
     name_attrs = ('ordered', 'partial')
 
-    def __init__(self, *expected_mapping, **expected_items):
-        # py2 :-(
-        self.ordered = expected_items.pop('ordered', False)
-        self.partial = expected_items.pop('partial', False)
-        self.recursive = expected_items.pop('recursive', False)
+    def __init__(
+            self,
+            *expected_mapping: tuple[Any, Any] | Mapping[Any, Any],
+            ordered: bool = False,
+            partial: bool = False,
+            recursive: bool = False,
+            **expected_items: Any,
+    ):
+        self.ordered = ordered
+        self.partial = partial
+        self.recursive = recursive
 
         if len(expected_mapping) == 1:
             expected = OrderedDict(*expected_mapping)
         else:
-            expected = OrderedDict(expected_mapping)
+            expected = OrderedDict(expected_mapping)  # type: ignore[arg-type]
         expected.update(expected_items)
 
         self.expected = expected
 
     def body(self) -> str:
-        # this can all go away and use the super class once py2 is gone :'(
         parts = []
         text_length = 0
         for key, value in self.expected.items():
@@ -1114,7 +1135,7 @@ class MappingComparison(StatefulComparison):
             sep = ', '
         return sep.join(parts)
 
-    def __ne__(self, other) -> bool:
+    def __ne__(self, other: Any) -> bool:
         try:
             actual_keys = other.keys()
             actual_mapping = dict(other.items())
@@ -1185,8 +1206,8 @@ class StringComparison:
 
     :param flag_names: See the :ref:`examples <stringcomparison>`.
     """
-    def __init__(self, regex_source: str, flags: int = None, **flag_names: str):
-        args = [regex_source]
+    def __init__(self, regex_source: str, flags: int | None = None, **flag_names: str):
+        args: list[Any] = [regex_source]
 
         flags_ = []
         if flags:
@@ -1197,23 +1218,23 @@ class StringComparison:
 
         self.re = re.compile(*args)
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         if not isinstance(other, str):
             return False
         if self.re.match(other):
             return True
         return False
 
-    def __ne__(self, other) -> bool:
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
     def __repr__(self) -> str:
         return '<S:%s>' % self.re.pattern
 
-    def __lt__(self, other) -> bool:
+    def __lt__(self, other: Any) -> bool:
         return self.re.pattern < other
 
-    def __gt__(self, other) -> bool:
+    def __gt__(self, other: Any) -> bool:
         return self.re.pattern > other
 
 
@@ -1231,13 +1252,13 @@ class RoundComparison:
         self.rounded = round(value, precision)
         self.precision = precision
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         other_rounded = round(other, self.precision)
         if type(self.rounded) is not type(other_rounded):
             raise TypeError('Cannot compare %r with %r' % (self, type(other)))
         return self.rounded == other_rounded
 
-    def __ne__(self, other) -> bool:
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
     def __repr__(self) -> str:
@@ -1253,15 +1274,158 @@ class RangeComparison:
 
     :param upper_bound: the inclusive upper bound for the acceptable range.
     """
-    def __init__(self, lower_bound, upper_bound):
+    def __init__(self, lower_bound: Any, upper_bound: Any) -> None:
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         return self.lower_bound <= other <= self.upper_bound
 
-    def __ne__(self, other) -> bool:
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
     def __repr__(self) -> str:
         return '<Range: [%s, %s]>' % (self.lower_bound, self.upper_bound)
+
+T = TypeVar('T')
+
+def like(t: type[T], **attributes: Any) -> T:
+    """
+    Create a type-safe partial comparison for use in strictly typed code.
+
+    This is a convenience function that creates a :class:`Comparison` with
+    ``partial=True`` but is typed to return the type being compared, making it
+    compatible with strict type checkers like mypy.
+
+    :param t: The type to compare against.
+    :param attributes: Keyword arguments specifying the attributes to check.
+    :return: A :class:`Comparison` object typed as the input type.
+    """
+    return Comparison(t, attribute_dict=attributes, partial=True)  # type: ignore[return-value]
+
+
+S = TypeVar("S", bound=Sequence[Any])
+S_ = TypeVar("S_", bound=Sequence[Any])
+
+
+@overload
+def sequence(
+    partial: bool = False,
+    ordered: bool = True,
+    recursive: bool = True,
+) -> Callable[[S], S]: ...
+
+
+@overload
+def sequence(
+    partial: bool = False,
+    ordered: bool = True,
+    recursive: bool = True,
+    *,
+    returns: type[S_],
+) -> Callable[[S], S_]: ...
+
+
+def sequence(
+    partial: bool = False,
+    ordered: bool = True,
+    recursive: bool = True,
+    *,
+    returns: type[S_] | None = None,
+) -> Callable[[S], S | S_]:
+    """
+    Create a type-safe sequence comparison with configurable partial matching
+    and ordering requirements.
+
+    This function returns a callable that wraps a sequence in a comparison object,
+    making it compatible with strict type checkers.
+
+    :param partial: If ``True``, only items in the expected sequence need to be present
+                    in the actual sequence. Defaults to ``False``.
+    :param ordered: If ``True``, items must appear in the same order. Defaults to ``True``.
+    :param recursive: If ``True``, provide detailed recursive comparison when differences
+                      are found. Defaults to ``True``.
+    :param returns: Optional type hint for the return type, used to satisfy type checkers
+                    when the comparison needs to appear as a different sequence type.
+    :return: A callable that takes a sequence and returns a comparison object typed
+             as a sequence.
+    """
+    def maker(items: S) -> S | S_:
+        return SequenceComparison(  # type: ignore[return-value]
+            *items, partial=partial, ordered=ordered, recursive=recursive
+        )
+
+    return maker
+
+
+@overload
+def contains(
+    items: S,
+) -> S: ...
+
+
+@overload
+def contains(
+    items: S,
+    *,
+    returns: type[S_],
+) -> S_: ...
+
+
+def contains(
+    items: S,
+    *,
+    returns: type[S_] | None = None,
+) -> S | S_:
+    """
+    Create a type-safe partial sequence comparison that ignores order.
+
+    Checks that the specified items are present in the actual sequence, regardless
+    of their order or what other items are present. This is useful when you only
+    care that certain elements exist in a collection.
+
+    :param items: The sequence of items that must be present.
+    :param returns: Optional type hint for the return type, used to satisfy type checkers
+                    when the comparison needs to appear as a different sequence type.
+    :return: A comparison object typed as a sequence.
+    """
+    return SequenceComparison(  # type: ignore[return-value]
+        *items, ordered=False, partial=True, recursive=True
+    )
+
+
+@overload
+def unordered(
+    items: S,
+) -> S: ...
+
+
+@overload
+def unordered(
+    items: S,
+    *,
+    returns: type[S_],
+) -> S_: ...
+
+
+def unordered(
+    items: S,
+    *,
+    returns: type[S_] | None = None,
+) -> S | S_:
+    """
+    Create a type-safe sequence comparison that ignores order but requires all
+    items to match.
+
+    Checks that the actual sequence contains exactly the same items as specified,
+    but in any order. This is useful when order doesn't matter but you want to
+    ensure no extra or missing items.
+
+    :param items: The sequence of items that must match exactly.
+    :param returns: Optional type hint for the return type, used to satisfy type checkers
+                    when the comparison needs to appear as a different sequence type.
+    :return: A comparison object typed as a sequence.
+    """
+    return SequenceComparison(  # type: ignore[return-value]
+        *items, ordered=False, partial=False, recursive=True
+    )
